(*
This file is part of the Kind verifier

* Copyright (c) 2007-2011 by the Board of Trustees of the University of Iowa, 
* here after designated as the Copyright Holder.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the University of Iowa, nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER ''AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*)

(** Process scheduler *)

(** 
@author Temesghen Kahsai

*)

open Types
open Flags
open Exceptions
open Channels
open Globals
open Mpi



module Dispatch =
struct    	  
  (** Only two processes. No Invariant generation *)
  let two_proc filename = 
    if(proc_rank = base_proc) then (* Base process *)
      begin 
	Globals.base_time_start := (wtime());
	print_to_user_final ("\n\n==========================================\n");
	print_to_user_final ("  PARALLEL KIND (NO INVARIANT GENERATION)\n");
	print_to_user_final ("==========================================\n\n");
	if !Flags.xml then 
	  print_to_user_final ("  KIND results will be saved in: " ^ filename ^ ".xml");
	Base_proc.main filename;
	(* If XML is activated terminate the tag for results*)
	if !Flags.xml then (
	  print_xml ("</Results>")
	)
      end
    else  (* Inductive process *)
      begin
	Globals.step_time_start := (wtime());
	Induct_proc.main filename;
      end
		
  (** Three process. With invariant generation. *)
  let three_proc filename =
    if(proc_rank = base_proc) then (* Base process *)
      begin 
	Globals.base_time_start := (wtime());
	    print_to_user_final ("\n\n==========================================\n");
	    print_to_user_final ("  PARALLEL KIND (WITH INVARIANT GENERATION)\n");
	    print_to_user_final ("==========================================\n\n");
	     if !Flags.xml then 
	       print_to_user_final ("  KIND results will be saved in: " ^ filename ^ ".xml\n");
	     Base_proc.main filename;
	     (* If XML is activated terminate the tag for results*)
	     if !Flags.xml then (
	       print_xml ("</Results>");
	     );
      end
    else if(proc_rank=step_proc) then
      begin
	Globals.step_time_start := (wtime());
	Induct_proc.main filename;
      end
    else
      begin
	Globals.inv_gen_time_start := (wtime());
	let defdoc,maxdepth,def_hash, pvars = Defgen.start_invariant_generator filename in 
	  if(!Flags.incremental) then (
	    Incremental_inv_gen.loop defdoc maxdepth def_hash pvars;
	  ) else (
	    Inv_generator.loop defdoc maxdepth def_hash pvars; 
	  );    
      end

  (** Schedule processes *)
  let schedule file proc_number = 
    match proc_number with 
	2 -> two_proc file
      | 3 -> three_proc file
      | _ -> raise Wrong_number_of_proc	
end  
  
  
  










